import 'dart:convert';

import 'dart:io';

const header = '''
class Produk {
  static const String table = 'm_product';
  static const String id = 'p_id';
  static const String satuan = 'p_satuan';
  static const String nama = 'p_nama';

  static tables() {
    return TableDatabase(
      tableName: table,
      foreignKey: ForeignKey(
        baseColumn: satuan,
        referenceColumn: Satuan.id,
        referenceTable: Satuan.table,
        deleteAction: ForeignKeyAction.setCascade,
        updateAction: ForeignKeyAction.setCascade,
      ),
      column: [
        ColumnDatabase(name: id, typeData: DataType.integer, primaryKey: true, autoIncrement: true),
        ColumnDatabase(name: nama, typeData: DataType.text),
        ColumnDatabase(name: satuan, typeData: DataType.integer),
      ],
    );
  }

  static add() async {
    final q = DB.table(table);
    q.data({nama: 'nice', satuan: '1'});
    await q.insert();
  }

  static get() async {
    final q = DB.table(table);
    print(await q.get());
  }
}

''';

const imports = '''import 'package:sql_query/builder/query_builder.dart';
import 'package:sql_query/database_model.dart';''';

class Query {
  String data = '';
  String key;
  Map value;
  Query({required this.key, required this.value});
  header() {
    String head = '';
    for (var i in key.split('_')) {
      head += i[0].toUpperCase() + i.substring(1, i.length);
    }
    data += '$imports\n\nclass $head';
  }

  newIndent() {
    data += ' { \n';
  }

  attribute() {
    data += "    static const String table = '$key';\n";
    value.forEach((k, v) {
      String parseKey = stringToUpper(k);

      data += "    static const String $parseKey = '$k';\n";
    });
    data += '\n';
  }

  closeIndent() {
    data += '} \n';
  }

  startBuildTable() {
    data += '    static tables()';
    newIndent();
    data += '        return TableDatabase(\n';
    data += '            tableName: "$key",\n';

    data += '            column: [\n';

    value.forEach((k, v) {
      final parseKey = stringToUpper(k);
      String insertData = 'typeData:';
      if (v.toString().toLowerCase().contains('integer')) {
        insertData += 'DataType.integer';
      } else if (v.toString().toLowerCase().contains('string')) {
        insertData += 'DataType.text';
      } else if (v.toString().toLowerCase().contains('double')) {
        insertData += 'DataType.double';
      }
      if (v.toString().toLowerCase().contains('primary_key')) {
        insertData += ', primaryKey: true';
      }
      if (v.toString().toLowerCase().contains('auto_increment')) {
        insertData += ', autoIncrement: true';
      }

      data += "                ColumnDatabase(name: $parseKey, $insertData),\n";
    });
    data += "            ],\n";
    data += "        );\n";
    data += "    ";
    closeIndent();
    data += '\n';
  }

  startBuildGet() {
    data += '    static get()';
    newIndent();
    data += '        final q = DB.table(table);\n';
    data += "        return q.get();\n";
    data += "    ";
    closeIndent();
    data += '\n';
  }

  startBuildInsert() {
    data += '    static insert()';
    newIndent();
    data += '        final q = DB.table(table);\n';
    data += "        // q.data(${rebuildMap(value)}); // add your data here\n";
    data += "        return q.insert();\n";
    data += "    ";
    closeIndent();
    data += '\n';
  }

  String build() {
    return data;
  }
}

String stringToUpper(value) {
  String parseKey = '';
  for (var i in value.toString().toLowerCase().replaceAll(' ', '_').split('_')) {
    if (parseKey == '') {
      parseKey += i;
    } else {
      if (i.length > 1) {
        parseKey += i[0].toUpperCase() + i.substring(1, i.length);
      } else {
        parseKey += i.toUpperCase();
      }
    }
  }
  return parseKey;
}

Map rebuildMap(data) {
  Map rebuild = {};
  data.forEach((k, v) {
    if (rebuild[k.replaceAll(' ', '_')] == null) {
      rebuild.addAll({
        stringToUpper(k): '"your value"',
      });
    }
  });
  return rebuild;
}

savePath(String path, String base) async {
  final slice = path.split('/');
  String dirs = '';
  for (int i = 0; i < slice.length; i++) {
    dirs += slice[i];
    if (i < slice.length - 1) {
      dirs += '/';
    }
    try {
      await Directory(base + dirs).create().then((Directory directory) {});
    } catch (_) {}
  }
}
